// Butterfly Curve (Parametric Equation)
// By: John Riselvato - June 5th, 2012
// kite.freeshell.org

#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include <stdio.h>
double animation = 0.0;

void lightDisplay(void){
  GLfloat mat_spec[] = {0.0, 0.0, 0.1, 1.0};
  GLfloat mat_shininess[] = {100.0};
  GLfloat light_position[] = {0.0, 1.0, 0.6, 1.0};
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_spec);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
}

void displayForGlut(void){
  glClear(GL_COLOR_BUFFER_BIT);
  lightDisplay();
  int i;
  double e = 2.71828182845904523536;
  double power;
  double green, blue; 
  double x, y, z;
  glBegin(GL_POINTS);
  for(i = 0; i < 10000; i++){
    power = pow(e, cos(i));
    x = sin(i)*(power-2*cos(4*i)-(sin(i/12)*sin(i/12)*sin(i/12)*sin(i/12)*sin(i/12)));
    y = cos(i)*(power-2*cos(4*i)-(sin(i/12)*sin(i/12)*sin(i/12)*sin(i/12)*sin(i/12)));
		if (x < 0.5 && x > -0.5){
			if(i%2 == 0){
				z = (x*x)-0.25;
			} else {
				z = -(x*x)+0.25;
			}
		} else {
    	z = animation*(x*x);
		}
    green = cos(x/y);
    blue = cos(sin(green*green));
    glColor3f(0.0, blue, green);
    glVertex3f(x, y, z);
  }
  glEnd();
  glFlush();
}

void Keyboard(unsigned char key, int x, int y){
  switch(key){
    case 'w':
      glTranslatef(0.5, 0.0, 0.0);
      glRotatef(10.0, 0.5, 0.0, 0.0);
      glTranslatef(-0.5, 0.0, 0.0);
      break;
    case 's':
      glTranslatef(0.5, 0.0, 0.0);
      glRotatef(-10.0, 0.5, 0.0, 0.0);
      glTranslatef(-0.5, 0.0, 0.0);
      break;
    case 'a':
      glTranslatef(0.5, 0.0, 0.0);
      glRotatef(10.0, 0.0, 0.0, 0.5);
      glTranslatef(-0.5, 0.0, 0.0);
      break;
    case 'd':
      glTranslatef(0.5, 0.0, 0.0);
      glRotatef(-10.0, 0.0, 0.0, 0.5);
      glTranslatef(-0.5, 0.0, 0.0);
      break;
    case 'z':
      if (animation < 0.1)
        animation = animation + 0.05;
      else
        animation = animation - 0.05;
      break;
		case 'i':
			glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
    	glFrustum(-5.0, 5.0, -5.0, 5.0, -5.0, 5.0);
			break;
  }
  glutPostRedisplay();
}

int initGlutDisplay(int argc, char* argv[]){
	puts("igd");
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB);
  glutInitWindowSize(600, 600);
  glutInitWindowPosition(100, 100);
  glutCreateWindow("Example 1.0: Hello World!");
  glMatrixMode(GL_PROJECTION);
  
	//Setting up lighting
	glDepthFunc(GL_LEQUAL);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	//setting depth
	glClearDepth(1.0);
	glEnable(GL_DEPTH_TEST);

	glLoadIdentity();
  glOrtho(-5.0, 5.0, -5.0, 5.0, -5.0, 5.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glutKeyboardFunc(Keyboard);  
  glutDisplayFunc(displayForGlut);
  glutMainLoop();
  return 0;
}

int main(int argc, char* argv[]){
	puts("main");
  initGlutDisplay(argc, argv);
}
